## Copyright (C) 2006 Bill Denney
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
## 02110-1301, USA.

## -*- texinfo -*-
## @deftypefn {Function File} {} @var{h} = __uiobject_create_object__ (@var{type}, @var{prop}, @var{val}, @dots{})
## @deftypefnx {Function File} {} @var{h} = __uiobject_create_object__ (@var{type}, @var{propstruct})
## Create a new uiobject of type @var{type} returning its handle @var{h}
## with property @var{prop} and value @var{val} (or multiple of each).
## The properties and values may also be given as a structure. This
## should be used to create any type of object except for the root
## object (root is automatically created when using this function if it
## has not yet been created).
## @end deftypefn

## Author: Bill Denney

function h = __uiobject_create_object__ (objtype, varargin)

  ## check the input
  if ((nargin == 2) && ~ isstruct (varargin{1}))
    error ("__uiobject_create_object__: if a structure is given, it must be the only additional arguement");
  elseif ((nargin > 2) && (mod (nargin, 2) ~= 1))
    usage ("__uiobject_create_object__ (objtype, prop, val, ...)");
  elseif strcmpi (objtype, "root")
    error ("__uiobject_create_object__: cannot create the root object");
  endif

  ## normalize the input
  objtype = lower (objtype);

  __uiobject_globals__;

  ## find the first empty index for the object
  idx = find ([__uiobject_list_status__.in_use] == false, 1);
  if isempty (idx)
    idx = length (__uiobject_list__) + 1;
    __uiobject_grow_list__ ();
  endif

  s = get (0, ["Default." objtype]);
  s.timestamp = __uiobject_update_timestamp__ ();

  handles = ...
      [__uiobject_list_status__([__uiobject_list_status__.in_use] == true).handle];
  if isfield (s, "currentaxes")
    ## it is a figure-type object
    h = max (handles) + 1;
  else
    ## it is another type of object
    h = min (handles) - 1;
  endif
  __uiobject_list_status__(idx).handle = h;

  if (~ isempty (varargin))
    ## convert from strings to struct if necessary
    if ischar (varargin{1})
      for i = 1:2:length (varargin)
	snew.(varargin{i}) = varargin{i+1};
      endfor
    elseif isstruct (varargin{1})
      snew = varargin{1};
    else
      error ("__uiobject_create_object__: invalid class for first arguement");
    endif

    ## set and check the given properties (we assume that all the default
    ## properties are already good).
    fields = lower (fieldnames (snew));
    for i = 1:length (fields)
      [val, err] = __uiobject_validate__ (objtype, fields{i}, snew.(fields{i}));
      if isempty (err)
	s.(fields{i}) = snew.(fields{i});
      else
	error ("__uiobject_create_object__: %s's property %s %s", ...
	       objtype, fields{i}, err);
      endif
    endfor
  endif

  ## check to make sure that the required variables are set
  required = get (0, ["Required." objtype]);
  for i = 1:length(required)
    if isempty (s.(required{i}))
      error ("__uiobject_create_object__: creation of %s missing required field %s", ...
	     objtype, required{i});
    endif
  endfor

  ## put the new object into its place
  __uiobject_list__{idx} = s;
  ## only set this to in use after everything has succeeded
  __uiobject_list_status__(idx).in_use = true;

endfunction