## Copyright (C) 2006 Bill Denney
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
## 02110-1301, USA.

## -*- texinfo -*-
## @deftypefn {Function File} {[@var{marker}, @var{line}, @var{color}, @var{legend}]} = __uiobject_parse_fmt__ (@var{fmt})
## Parse the values out of the @var{fmt} string.  @var{fmt} may be a
## standard format string, many format strings separated by one or more
## pipes (|), or many format strings in a cellstr array.
##
## Warning: if a color is not specified, and new style format is used,
## an axis will be created.
## @seealso{plot}
## @end deftypefn

## Author: Bill Denney

function [mark, lin, color, leg] = __uiobject_parse_fmt__ (fmt)

  if (nargin ~= 1)
    usage ("[marker, line, color, legend] = __uiobject_parse_fmt__ (fmt)");
  elseif iscellstr (fmt)
    ## recurse for input of a cellstr of formats
    mark = cell (size (fmt,1), size (fmt,2));
    lin = cell (size (fmt,1), size (fmt,2));
    color = cell (size (fmt,1), size (fmt,2));
    leg = cell (size (fmt,1), size (fmt,2));
    for i = 1:numel (fmt)
      [mark{i}, lin{i}, color{i}, leg{i}] = __uiobject_parse_fmt__ (fmt{i});
    endfor

  elseif (~ ischar (fmt))
    ## give an error if it's not a character string
    error ("__uiobject_parse_fmt__: format must be a character string or cellstr");

  elseif (~ isempty (findstr (fmt, "|")))
    ## recurse for multiple formats separated by pipes
    pipes = [0 findstr(fmt, "|") (length(fmt)+1)];
    for i = 1:length (pipes) - 1
      [mark{i}, lin{i}, color{i}, leg{i}] = ...
	  __uiobject_parse_fmt__ (fmt(pipes(i)+1:pipes(i+1)-1));
    endfor
  endif

  origfmt = fmt;

  ## clip out the legend if it's there first to make the rest easier
  legend_ends = find (fmt == ";");
  if (~ isempty (legend_ends))
    if (length (legend_ends) == 2)
      if (legend_ends(2) == length (fmt))
	leg = {fmt(legend_ends(1)+1:legend_ends(2)-1)};
	fmt = fmt(1:legend_ends(1)-1);
      else
	error ("__uiobject_parse_fmt__: legend must come last in the format");
      endif
    else
      error ("__uiobject_parse_fmt__: invalid legend in format");
    endif
  else
    leg = {""};
  endif

  fmtnum = str2num(fmt);
  if (~ isempty (fmtnum))
    ## some support for older gnuplot-ish formats
    if (fmtnum > 10)
      colornum = floor (fmtnum/10);
      pointnum = mod (fmtnum, 10);
    elseif (fmtnum > 0)
      colornum = fmtnum;
      pointnum = [];
    else
      error ("__uiobject_parse_fmt__: number style cannot be negative");
    endif

    switch colornum
      case 1
	## red
	color = "r";
      case 2
	## green
	color = "g";
      case 3
	## blue
	color = "b";
      case 4
	## magenta
	color = "m";
      case 5
	## cyan
	color = "c";
      case 6
	## brown
	## this is HTML's definition of brown, so it should be good
	## enough
	color = [0.5 0.25 0 1];
      otherwise
	error ("__uiobject_parse_fmt__: color number style must be an integer in the range 1 to 6");
    endswitch

    switch pointnum
      case 1
	mark = "*";
      case 2
	mark = "+";
      case 3
	mark = "o";
      case 4
	mark = "x";
      case 5
	error ("__uiobject_parse_fmt__: house point style is not supported, please try one of our other beautiful styles");
      case 6
	error ("__uiobject_parse_fmt__: there exists point style is not supported, please try one of our other beautiful styles");
      otherwise
	## make it the default marker-- none
	mark = "none";
    endswitch

  else
    ## new style formatting

    ## find the marker
    markfmts = {"+" "o" "*" "x" "."};
    i = 0;
    mark = "";
    while (isempty (mark) && (i < length (markfmts)))
      i = i + 1;
      markloc = findstr (fmt, markfmts{i}, 0);

      if (length (markloc) > 1)
	error ("__uiobject_parse_fmt__: too many marker definitions in %s", ...
	       origfmt);
      elseif (~ isempty (markloc))
	mark = fmt(markloc(1):(markloc(1) + length (markfmts{i}) - 1));
      endif
    endwhile

    if isempty (mark)
      ## If the color was not set, grab the color from the current axis
      ## and rotate the color order.
      mark = get (gca (), "MarkerOrder");
      set (gca (), "MarkerOrder", mark([2:length(mark) 1]));
      mark = mark(1);
    endif

    ## find the line style
    linefmts = {"--" "-." ":" "-"};
    i = 0;
    lin = "";
    while (isempty (lin) && (i < length (linefmts)))
      i = i + 1;
      lineloc = findstr (fmt, linefmts{i}, 0);

      if (length (lineloc) > 1)
	error ("__uiobject_parse_fmt__: too many line definitions in %s", ...
	       origfmt);
      elseif (~ isempty (lineloc))
	lin = fmt(lineloc(1):(lineloc(1) + length (linefmts{i}) - 1));
	## clip out the line format
	fmt = fmt([1:lineloc(1)-1 ...
		   (lineloc(1) + length (linefmts{i}):length(fmt))]);
      endif
    endwhile

    if isempty (lin)
      ## If the color was not set, grab the color from the current axis
      ## and rotate the color order.
      lin = get (gca (), "LineOrder");
      set (gca (), "LineOrder", lin([2:size(lin,1) 1]));
      lin = lin{1};
    endif

    ## find the color
    colorfmts = {"r" "g" "b" "c" "m" "y" "k" "w"};
    i = 0;
    color = "";
    while (isempty (color) && (i < length (colorfmts)))
      i = i + 1;
      colorloc = findstr (fmt, colorfmts{i}, 0);
      if (length (colorloc) > 1)
	error ("__uiobject_parse_fmt__: too many color definitions in %s", ...
	       origfmt);
      elseif (~ isempty (colorloc))
	color = fmt(colorloc(1):(colorloc(1) + length (colorfmts{i}) - 1));
	## clip out the color format
	fmt = fmt([1:colorloc(1)-1 ...
		   (colorloc(1) + length (colorfmts{i}) - 1)]);
      endif
    endwhile

    if isempty (color)
      ## If the color was not set, grab the color from the current axis
      ## and rotate the color order.
      color = get (gca (), "ColorOrder");
      set (gca (), "ColorOrder", color([2:size(color,1) 1], :));
      color = color(1,:);
    endif
    [color, err] = __uiobject_valid_color__ ([], [], color);

  endif

  ## we're finally done with this fun task-- parsing is always fun
endfunction

## Tests
## Note that incomplete formats aren't tested because they would require
## creating a figure and associated objects.

## The linestyle "-" with the marker "." can be ambiguous if it is
## specified with the order "-." because this is a line style.  Because
## of this, markers are found first and lines are found second to avoid
## this issue when possible.  The "." marker is detected last of the
## marker styles.

%!assert(__uiobject_parse_fmt__ ("--.r.;abcd'"), ".", "--", [1 0 0 1], "abcd")
## this should give an error because the legend isn't last
## %!assert(__uiobject_parse_fmt__ ("--.r.;abcd'"), ".", "--", [1 0 0 1], "abcd")
## testing format order
%!assert(__uiobject_parse_fmt__ ("--.r"), ".", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ ("--r."), ".", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ ("r--."), ".", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ ("r.--"), ".", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ (".r--"), ".", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ (".--r"), ".", "--", [1 0 0 1], "")

## test all line formats
%!assert(__uiobject_parse_fmt__ (".--r"), ".", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ (".-r"), ".", "-", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ (".:r"), ".", ":", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ (".-.r"), ".", "-.", [1 0 0 1], "")

## test all colors
%!assert(__uiobject_parse_fmt__ (".--r"), ".", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ (".--g"), ".", "--", [0 1 0 1], "")
%!assert(__uiobject_parse_fmt__ (".--b"), ".", "--", [0 0 1 1], "")
%!assert(__uiobject_parse_fmt__ (".--c"), ".", "--", [0 1 1 1], "")
%!assert(__uiobject_parse_fmt__ (".--m"), ".", "--", [1 0 1 1], "")
%!assert(__uiobject_parse_fmt__ (".--y"), ".", "--", [1 1 0 1], "")
%!assert(__uiobject_parse_fmt__ (".--k"), ".", "--", [0 0 0 1], "")
%!assert(__uiobject_parse_fmt__ (".--w"), ".", "--", [1 1 1 1], "")

## test all markers
%!assert(__uiobject_parse_fmt__ (".--r"), ".", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ ("+--r"), "+", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ ("o--r"), "o", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ ("*--r"), "*", "--", [1 0 0 1], "")
%!assert(__uiobject_parse_fmt__ ("x--r"), "x", "--", [1 0 0 1], "")

## add a legend
%!assert(__uiobject_parse_fmt__ (".--r;abc;"), ".", "--", [1 0 0 1], "abc")