## Copyright (C) 2005, 2006 John W. Eaton
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
## 02110-1301, USA.

## -*- texinfo -*-
## @deftypefn {Function File} {} axis ([@var{xmin} @var{xmax} @var{ymin} @var{ymax}])
## @deftypefnx {Function File} {} axis ([@var{xmin} @var{xmax} @var{ymin} @var{ymax} @var{zmin} @var{zmax} @var{cmin} @var{cmax}])
## @deftypefnx {Function File} {} axis auto
## @deftypefnx {Function File} {} axis manual
## @deftypefnx {Function File} {} axis tight
## @deftypefnx {Function File} {} axis fill
## @deftypefnx {Function File} {} axis ij
## @deftypefnx {Function File} {} axis xy
## @deftypefnx {Function File} {} axis equal
## @deftypefnx {Function File} {} axis image
## @deftypefnx {Function File} {} axis square
## @deftypefnx {Function File} {} axis vis3d
## @deftypefnx {Function File} {} axis normal
## @deftypefnx {Function File} {} axis off
## @deftypefnx {Function File} {} axis on
## @deftypefnx {Function File} {} axis (@var{h}, @dots{})
## @deftypefnx {Function File} {[mode, visibility, direction]} axis ("state")
## The options mean
## @itemize @samp
## @item auto
## Automatically set the axis limits.
## @item manual
## Manually set the axis limits.
## @item tight
## Set the axis limits to the range of the data exactly.
## @item fill
## Set the aspect ratio to fill the current axis box.
## @item ij
## Place the system origin in the upper left corner with values increasing
## top to bottom, left to right.  This is useful for images.
## @item xy
## Place the system origin in the lower left corner with the values
## increasing from bottom to top, left to right.
## @item equal
## Set the aspect ratio so that the units are equal in each direction.
## @item image
## Same as the combination of equal and tight.
## @item square
## Make the axis box square (or a cube for 3d).
## @item vis3d
## Not supported.
## @item normal
## Does one-time automatic fitting while leaving the manual settings set
## to manual.
## @item off
## Turn off axis lines, ticks, and labels.
## @item on
## Turn on axis lines, ticks, and labels.
## @end itemize
## @seealso{axes,set,get}
## @end deftypefn

## Author: Bill Denney

## PKG_ADD: mark_as_command axis

function varargout = axis (a1, a2)

  ## classify the input arguements
  if (nargin > 2)
    usage ("axis (...)");
  elseif all (ishandle (a1(:)))
    ha = a1;
    setting = a2;
  elseif ((nargin == 1) && (ischar (a1) || isnumeric (a1)))
    ha = gca ();
    setting = a1;
  else
    error ("axis: invalid input");
  endif

  ## more input checking and the processing
  if ischar (setting)
    switch lower (setting)
      case {"auto"}
	
      case {"manual"}
      case {"tight"}
      case {"fill"}
      case {"ij"}
      case {"xy"}
      case {"equal"}
      case {"image"}
      case {"square"}
      case {"vis3d"}
      case {"normal"}
      case {"off"}
      case {"on"}
      case {"state"}
      otherwise
	error ("axis: invalid setting");
    endswitch
  elseif (isnumeric (setting) && ...
	  ((size (setting, 1) == 1) && ...
	   ((size (setting, 2) == 4) || (size (setting, 2) == 8))))
    ## we could add additional test for x, y, and z min being less than
    ## the respective max
    settings = {"xlim" setting(1:2) "ylim" setting(3:4)};
    if (length (setting) > 4)
      settings{5} = "zlim";
      settings{6} = setting(5:6);
    endif
  else
    error ("axis: invalid ranges");
  endif

  ## put all the settings into the fields
  set (ha, settings{:});

  ## return some stuff
  if (nargout == 1) || (nargout == 3)
    xlim = get (ha, "xlim");
    ylim = get (ha, "ylim");
    zlim = get (ha, "zlim");
    retval = [xlim ylim zlim];
    if nargout == 1
      varargout{1} = retval;
    else
      varargout = {xlim ylim zlim};
    endif
  endif

endfunction
