## Copyright (C) 2006 Bill Denney
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
## 02110-1301, USA.

## -*- texinfo -*-
## @deftypefn {Function File} {} get (@var{h}, @var{p})
## @deftypefnx {Function File} {} get (@var{h})
## Return the named property @var{p} from the graphics handle @var{h}.
## If @var{p} is omitted, return the complete property list for @var{h}.
## @var{h} may be a vector, and @var{p} may be a cellstr, if either of
## these are vectors, then the output will be a cell array with size
## [numel(@var{h}) numel(@var{p})].
## @seealso{set}
## @end deftypefn

## Author: Bill Denney

function retval = get (h, property)

  __uiobject_globals__;

  if ((nargin < 1) || (nargin > 2))
    print_usage ();
  endif

  if any (~ ishandle (h))
    error ("get: invalid handle");
  endif

  idx = __uiobject_handle2idx__ (h);
  objs = __uiobject_list__(idx);

  if (nargin == 1)
    ## the simple case
    retval = objs(:);

  else
    if ischar (property)
      property = {property};
    elseif ~ iscellstr (property)
      error ("get: property must be a string or a cellstr");
    endif
    key = lower (property(:));

    ## determine if any of the keys are special (i.e. we will have to
    ## recurse structures for them).
    periods = cell (size (key));
    if any (h == 0)
      ## there is no special handling for any but root objects
      for i = 1:numel (key)
	periods{i} = strfind (key{i}, ".");
      endfor
    endif

    for i = 1:numel (objs)
      obj = objs{i};
      for j = 1:numel (key)
	if ((h == 0) && ~ isempty(periods(j)))
	  ## special handling for special fields in the root object
	  deftypesep = [periods{i} (numel (key{j})+1)];
	  retval{i,j} = obj.(key{j}(1:deftypesep(1)-1));

	  for k = 1:length(deftypesep)-1
	    thisfield = key{j}(deftypesep(k)+1:deftypesep(k+1)-1);
	    if (isfield (retval{i,j}, thisfield))
	      ## delve within the special structure as far as necessary
	      retval{i,j} = retval{i,j}.(thisfield);
	    else
	      keyboard
	      error ("get: root object has no property named %s (failed at %s)", ...
		     property, thisfield);
	    endif
	  endfor

	elseif (isfield (obj, key{j}))
	  ## the normal case
	  retval{i,j} = obj.(key{j});
	else
	  error ("get: object type %s has no property named `%s'", ...
		 obj.type, property);
	endif
      endfor
    endfor
  endif

  ## Don't return a cell if only one return value is requested
  if (numel (retval) == 1)
    retval = retval{1};
  endif

endfunction
