## Copyright (C) 2006 Bill Denney
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
## 02110-1301, USA.

## -*- texinfo -*-
## @deftypefn {Function File} {} plot (@var{args})
## This function produces two-dimensional plots.  Many different
## combinations of arguments are possible.  The simplest form is
##
## @example
## plot (@var{y})
## @end example
##
## @noindent
## where the argument is taken as the set of @var{y} coordinates and the
## @var{x} coordinates are taken to be the indices of the elements,
## starting with 1.
##
## If more than one argument is given, they are interpreted as
##
## @example
## plot (@var{x}, @var{y}, @var{fmt} ...)
## @end example
##
## @noindent
## where @var{y} and @var{fmt} are optional, and any number of argument
## sets may appear.  The @var{x} and @var{y} values are
## interpreted as follows:
##
## @itemize @bullet
## @item
## If a single data argument is supplied, it is taken as the set of @var{y}
## coordinates and the @var{x} coordinates are taken to be the indices of
## the elements, starting with 1.
##
## @item
## If the first argument is a vector and the second is a matrix, the
## the vector is plotted versus the columns (or rows) of the matrix.
## (using whichever combination matches, with columns tried first.)
##
## @item
## If the first argument is a matrix and the second is a vector, the
## the columns (or rows) of the matrix are plotted versus the vector.
## (using whichever combination matches, with columns tried first.)
##
## @item
## If both arguments are vectors, the elements of @var{y} are plotted versus
## the elements of @var{x}.
##
## @item
## If both arguments are matrices, the columns of @var{y} are plotted
## versus the columns of @var{x}.  In this case, both matrices must have
## the same number of rows and columns and no attempt is made to transpose
## the arguments to make the number of rows match.
##
## If both arguments are scalars, a single point is plotted.
## @end itemize
##
## If the @var{fmt} argument is supplied, it is interpreted as
## follows.  If @var{fmt} is missing, the default gnuplot line style
## is assumed.
##
## @table @samp
## @item -
## Set lines plot style (default).
##
## @item .
## Set dots plot style.
##
## @item @@
## Set points plot style.
##
## @item -@@
## Set linespoints plot style.
##
## @item ^
## Set impulses plot style.
##
## @item L
## Set steps plot style.
##
## @item @var{n}
## Interpreted as the plot color if @var{n} is an integer in the range 1 to
## 6.
##
## @item @var{nm}
## If @var{nm} is a two digit integer and @var{m} is an integer in the
## range 1 to 6, @var{m} is interpreted as the point style.  This is only
## valid in combination with the @code{@@} or @code{-@@} specifiers.
##
## @item @var{c}
## If @var{c} is one of @code{"k"}, @code{"r"}, @code{"g"}, @code{"b"},
## @code{"m"}, @code{"c"}, or @code{"w"}, it is interpreted as the plot
## color (black, red, green, blue, magenta, cyan, or white).
##
## @item ";title;"
## Here @code{"title"} is the label for the key.
##
## @item +
## @itemx *
## @itemx o
## @itemx x
## Used in combination with the points or linespoints styles, set the point
## style.
## @end table
##
## The color line styles have the following meanings on terminals that
## support color.
##
## @example
## Number  Gnuplot colors  (lines)points style
##   1       red                   *
##   2       green                 +
##   3       blue                  o
##   4       magenta               x
##   5       cyan                house
##   6       brown            there exists
## @end example
##
## The @var{fmt} argument can also be used to assign key titles.
## To do so, include the desired title between semi-colons after the
## formatting sequence described above, e.g. "+3;Key Title;"
## Note that the last semi-colon is required and will generate an error if
## it is left out.
##
## Here are some plot examples:
##
## @example
## plot (x, y, "@@12", x, y2, x, y3, "4", x, y4, "+")
## @end example
##
## This command will plot @code{y} with points of type 2 (displayed as
## @samp{+}) and color 1 (red), @code{y2} with lines, @code{y3} with lines of
## color 4 (magenta) and @code{y4} with points displayed as @samp{+}.
##
## @example
## plot (b, "*")
## @end example
##
## This command will plot the data in the variable @code{b} will be plotted
## with points displayed as @samp{*}.
##
## @example
## t = 0:0.1:6.3;
## plot (t, cos(t), "-;cos(t);", t, sin(t), "+3;sin(t);");
## @end example
##
## This will plot the cosine and sine functions and label them accordingly
## in the key.
## @seealso{semilogx,semilogy,loglog,polar,mesh,contour,bar,stairs,errorbar,replot,xlabel,ylabel,zlabel,title}
## @end deftypefn

## Author: Bill Denney

function varargout = plot (varargin)

  ## x, y, and fmt are converted to cell vectors

  ## FIXME: these should come from the current axes
  ## Default styles
  default.fmt = "-.";

  ## check for erronious input before doing a full parsing
  arg = 0;
  ## the next arguement type:
  ## "x" must be an x value
  ## "y" must be a y value
  ## "fmt" must be either a format or an x value
  nxtarg = "x";
  values.x = {};
  values.y = {};
  values.fmt = {};

  ## parse values that are of the form x, y, and optionally fmt
  ## repeating
  while (arg < nargin)
    arg++;
    if strcmp (nxtarg, "x")
      if isnumeric (varargin{arg})
	values.x{length (values.x) + 1} = varargin{arg};
	nxtarg = "y";
      else
	error ("plot: arguement %d is not numeric (should be x values)", arg);
      endif
    elseif strcmp (nxtarg, "y")
      if isnumeric (varargin{arg})
	values.y{length (values.y) + 1} = varargin{arg};
	nxtarg = "fmt";
      else
	error ("plot: arguement %d is not numeric (should be y values)", arg);
      endif
    elseif strcmp (nxtarg, "fmt")
      if ischar (varargin{arg})
	values.fmt{length (values.fmt) + 1} = varargin{arg};
	nxtarg = "x";
      elseif isnumeric (varargin{arg})
	## empty will signify that the color, etc needs to be determined
	## automatically
	values.fmt{length (values.fmt) + 1} = "";
	values.x{length (values.x) + 1} = varargin{arg};
	nxtarg = "y";
      else
	error ("plot: arguement %d must be either numeric or character", arg);
      endif
    endif
  endwhile

  ## automatically give x values to match the length of the y values in
  ## case of one input.
  if (nargin == 1)
    values.y{1} = values.x{1};
    values.x{1} = (1:length (values.x{1}))';
  endif

  ## check for a single aruement and make it a y value instead of x
  if ((length (values.y) == 0) && (length (values.x) == 1))
    values.y = values.x;
    if isvector (values.y{1})
      values.x = 1:length (values.y{1});
    else
      values.x = 1:size (values.y{1}, 1);
    endif
  endif

  ## add on a final format arguement if necessary
  if (length (values.fmt) < length (values.x))
    values.fmt{length (values.fmt) + 1} = "";
  endif

  ## check for general consistency
  if (length (values.x) ~= length (values.y))
    error ("plot: there must be an equal number of x and y arguements");
  elseif (length (values.fmt) ~= length (values.x))
    error ("plot: there must be an equal number of x, y, and fmt arguements after parsing")
  endif

  ## check to make sure that all x and y values are 1 or 2 dimensional
  for i = 1:length (values.x)
    if ((size (values.x{i}) > 2) || (size (values.x{i}) == 0))
      error ("plot: invalid number of dimensions for x in plot %d", i);
    endif
    if ((size (values.y{i}) > 2) || (size (values.y{i}) == 0))
      error ("plot: invalid number of dimensions for y in plot %d", i);
    endif
  endfor

  ## check for size/length consistency in x, y args.  After this loop,
  ## all plots will be along the columns of the values.
  for i = 1:length (values.x)
    if isvector (values.x{i})
      ## make sure it's a column vector
      values.x{i} = reshape (values.x{i}, [], 1);
      if isvector (values.y{i})
	## make sure it's a column vector
	values.y{i} = reshape (values.y{i}, [], 1);
	if (length (values.x{i}) ~= length (values.y{i}))
	  error ("plot: vector lengths don't match on plot %d", i);
	endif
      else
	## y is a matrix
	if (size (values.y{i}, 1) == length (values.x{i}))
	  ## everything is fine as is
	elseif (size (values.y{i}, 2) == length (values.x{i}))
	  ## it needs to be transposed
	  values.y{i} = values.y{i}';
	else
	  error (["plot: neither the columns or rows of y equal the " ...
		  "length of x for plot %d"], i);
	endif
      endif

    elseif isvector (values.y{i})
      ## make sure it's a column vector
      values.y{i} = reshape (values.y{i}, [], 1);
      if isvector (values.x{i})
	## make sure it's a column vector
	values.x{i} = reshape (values.x{i}, [], 1);
	if (length (values.x{i}) ~= length (values.y{i}))
	  error ("plot: vector lengths don't match on plot %d", i);
	endif
      else
	## x is a matrix
	if (size (values.x{i}, 1) == length (values.y{i}))
	  ## everything is fine as is
	elseif (size (values.y{i}, 2) == length (values.x{i}))
	  ## it needs to be transposed
	  values.y{i} = values.y{i}';
	endif
      endif

    else
      ## they're both matrices
      if any (size (values.x{i}) ~= size (values.y{i}))
	error ("plot: the size of the x and y matrices must be the same");
      endif
    endif
  endfor

  ## Validate the formats (and change them to linespec, colorspec,
  ## markerspec, and possibly a legend)
  for i = 1:length (values.fmt)
    [values(i).marker, values(i).linestyle, values(i).color, ...
     values(i).legend] = ...
	__uiobject_parse_fmt__ (values.fmt{i});
  endfor

  ## Start plotting
  for i = 1:length (values.x)
    for j = 1:size (values(i).x, 2)
      ## plot all lines, grouping lines together when they're given as a group
      h(j) = line ("xdata", values(i).x(:,j), ...
		   "ydata", values(i).y(:,j), ...
		   "markeredgecolor", values(i).color(j,:), ...
		   "markerfacecolor", values(i).color(j,:), ...
		   "color", values(i).color(j,:), ...
		   "marker", values(i).marker(j), ...
		   "linestyle", values(i).linestyle(j), ...
		   "label", values(i).legend{j});
    endfor

    ## setup the return handles
    if (length (h) > 1)
      ## group the lines
      hg = group ();
      __uiobject_adopt__ (hg, h);
      retval(i) = hg;
    else
      retval(i) = h;
    endif
  endfor

  ## return something if requested
  if (nargout == 1)
    varargout{i} = retval;
  endif

endfunction